"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require("react");
var ReactDOM = require("react-dom");
var _ = require("underscore");
var icon_1 = require('./../display/icon');
var grid_1 = require('./../layout/grid');
var button_1 = require('./../interaction/button');
var DropdownSelect = (function (_super) {
    __extends(DropdownSelect, _super);
    function DropdownSelect() {
        _super.call(this);
        this.itemHeight = 50;
        this.state = { filteredOptions: [], query: "", open: false, selectedValue: null, selectedIndex: 0, remoteSearching: false, offsetIndex: 0 };
    }
    DropdownSelect.prototype.filterRemote = function (query) {
        var _this = this;
        if (this.timer) {
            window.clearTimeout(this.timer);
        }
        this.timer = window.setTimeout(function () {
            _this.setState({ remoteSearching: true });
            _this.props.remoteQuery(query).then(function (filteredOptions) {
                _this.setState({ filteredOptions: filteredOptions, remoteSearching: false });
            });
        }, this.props.remoteThrottle || 500);
    };
    DropdownSelect.prototype.filter = function (query) {
        var _this = this;
        var q = query.toLowerCase();
        if (this.props.remoteQuery) {
            this.filterRemote(query);
        }
        else {
            if (query.length <= this.props.minimumLength || 1) {
                this.setState({ filteredOptions: this.props.options, query: query }, function () { return _this.constrainIndex(); });
            }
            else {
                this.setState({ filteredOptions: _.reject(this.props.options, function (o) { return o.name.toLowerCase().indexOf(q) === -1; }), query: query }, function () { return _this.constrainIndex(); });
                ;
            }
        }
    };
    DropdownSelect.prototype.focusInput = function () {
        var _this = this;
        if (!this.state.open) {
            this.setState({ open: true }, function () {
                ReactDOM.findDOMNode(_this).querySelector("input").focus();
                document.addEventListener("click", _this, false);
            });
        }
        else {
            this.setState({ open: false, query: "", filteredOptions: this.props.options || [] });
            document.removeEventListener("click", this, false);
        }
    };
    DropdownSelect.prototype.handleEvent = function (e) {
        if (ReactDOM.findDOMNode(this).contains(e.target)) {
            return;
        }
        this.setState({ open: false, query: "", filteredOptions: this.props.options || [] });
        document.removeEventListener("click", this, false);
    };
    DropdownSelect.prototype.componentWillMount = function () {
        this.setState({ filteredOptions: this.props.options || [] });
    };
    DropdownSelect.prototype.checkKey = function (e) {
        var currentIndex = this.state.selectedIndex;
        if (e.keyCode === 40 && this.state.filteredOptions.length !== 0) {
            var offsetIndex = Math.min((this.props.visibleItems || 3) - 1, this.state.offsetIndex + 1);
            var selectedIndex = Math.min(this.state.selectedIndex + 1, this.state.filteredOptions.length - 1);
            var listElement = ReactDOM.findDOMNode(this).querySelector(".dropdown-select-list");
            this.setState({ offsetIndex: offsetIndex });
            if (offsetIndex >= 2) {
                listElement.scrollTop = (selectedIndex - 2) * this.itemHeight;
            }
            var selectedItem = this.state.filteredOptions[selectedIndex];
            this.setState({ selectedIndex: selectedIndex, query: selectedItem.name });
            e.preventDefault();
            return false;
        }
        if (e.keyCode === 38 && this.state.filteredOptions.length !== 0) {
            var offsetIndex = Math.max(this.state.offsetIndex - 1, 0);
            var selectedIndex = Math.max(this.state.selectedIndex - 1, 0);
            var listElement = ReactDOM.findDOMNode(this).querySelector(".dropdown-select-list");
            this.setState({ offsetIndex: offsetIndex });
            if (offsetIndex === 0) {
                listElement.scrollTop = (selectedIndex) * this.itemHeight;
            }
            var selectedItem = this.state.filteredOptions[selectedIndex];
            this.setState({ selectedIndex: selectedIndex, query: selectedItem.name });
            e.preventDefault();
            return false;
        }
        if (e.keyCode === 13 && this.state.filteredOptions.length !== 0) {
            var selectedValue = this.state.filteredOptions[this.state.selectedIndex];
            this.selectItem(selectedValue);
            e.preventDefault();
            return false;
        }
        this.filter(e.target.value);
    };
    DropdownSelect.prototype.constrainIndex = function () {
        if (this.state.selectedIndex > this.state.filteredOptions.length - 1) {
            this.setState({ selectedIndex: Math.max(this.state.filteredOptions.length - 1, 0) });
        }
    };
    DropdownSelect.prototype.buttonClick = function () {
        var selectedValue = this.state.filteredOptions[this.state.selectedIndex];
        this.selectItem(selectedValue);
    };
    DropdownSelect.prototype.selectItem = function (selectedValue) {
        this.setState({ selectedValue: selectedValue, open: false, query: "", filteredOptions: this.props.options || [], offsetIndex: 0 });
        document.removeEventListener("click", this, false);
        if (this.props.onSelected) {
            this.props.onSelected(selectedValue);
        }
    };
    DropdownSelect.prototype.render = function () {
        var _this = this;
        return (React.createElement("div", {className: "dropdown-select" + (this.props.className ? " " + this.props.className : '')}, React.createElement(grid_1.Grid, {className: "dropdown-value-display"}, React.createElement(grid_1.Row, null, React.createElement(grid_1.Col, {onClick: function () { return _this.focusInput(); }}, this.state.selectedValue ? React.createElement("div", null, this.state.selectedValue.name) : React.createElement("div", {className: "placeholder"}, this.props.placeholder)), this.state.selectedValue && this.props.canClear && React.createElement(grid_1.Col, {fixed: true, className: "clear-selected p-right-xsmall", onClick: function () { return _this.setState({ selectedValue: null, open: false, query: "", filteredOptions: _this.props.options || [] }); }}, React.createElement(icon_1.Icon, {icon: icon_1.Icon.Icomoon.cross})), this.props.hasGoButton && React.createElement(grid_1.Col, {fixed: true}, React.createElement(button_1.Button, {text: this.props.goButtonContent || "Go", className: "bg-positive", onClick: function () { return _this.buttonClick(); }})))), this.state.open &&
            React.createElement("div", {className: "dropdown-select-list-wrapper"}, React.createElement("input", {type: "text", value: this.state.query, onKeyUp: function (e) { return _this.checkKey(e); }, onChange: function (e) { return _this.setState({ query: e.target.value }); }, placeholder: "start typing to filter results..."}), this.state.remoteSearching && React.createElement(icon_1.Icon, {className: "spinner fg-info", icon: icon_1.Icon.Icomoon.spinner2}), React.createElement("div", {"data-id": "dropdown-select-list", className: "dropdown-select-list", style: { maxHeight: "" + (this.props.visibleItems || 3) * this.itemHeight }}, this.state.filteredOptions && this.state.filteredOptions.map(function (o, i) {
                return React.createElement("div", {"data-index": i, key: "dd-item-" + i, className: "dd-list-item" + (i === _this.state.selectedIndex ? ' selected' : ''), onClick: function () { return _this.selectItem(o); }}, o.name);
            }), this.state.filteredOptions.length === 0 && React.createElement("div", {className: "dd-list-item-no-select"}, "No results...")))));
    };
    return DropdownSelect;
}(React.Component));
exports.DropdownSelect = DropdownSelect;
